--[[
LuCI - Lua Configuration Interface

Copyright 2008 Steven Barth <steven@midlink.org>
Copyright 2008 Jo-Philipp Wich <xm@leipzig.freifunk.net>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

$Id$
]]--

local wa = require "luci.tools.webadmin"
local fs = require "nixio.fs"
local ttu  = require "luci.tools.ttutil"
local nw   = require "luci.tools.network"
local web  = require "luci.tools.webadmin"

arg[1] = arg[1] or ""

local has_3g    = fs.access("/usr/bin/gcom")
local has_pptp  = fs.access("/usr/sbin/pptp")
local has_pppd  = fs.access("/usr/sbin/pppd")
local has_pppoe = fs.glob("/usr/lib/pppd/*/rp-pppoe.so")()
local has_pppoa = fs.glob("/usr/lib/pppd/*/pppoatm.so")()
local has_docomo3g  = fs.access("/etc/init.d/docomo3g")
local has_emobile3g = fs.access("/etc/init.d/emobile")

local bridge_ifs = nw.bridge_ifs()
local single_ifs = nw.single_ifs()
local wifi_ifs   = nw.wireless_ifs()

m = Map("network", translate("interfaces_net"), translate("a_n_ifaces_tt"))
m:chain("wireless")

---------- function(s) --------------
-- args : ("eth1" or "eth0 eth1")
-- this function support the style "eth1 wlan0"
local function network_del_ifname(ifnames)
	if type(ifnames) == "string" then
	   ifnames = ttu.split(ifnames, " ")
	end
	for _, ifname in ipairs(ifnames) do
		m.uci:foreach("network", "interface",
			function(s)
				if s.ifname == ifname then
					m.uci:delete("network", s['.name'], "ifname")
				end
			end)
	end
end

-- args : (ADHOC_0)
local function wifi_del_network(net)
	local saved = false
	m.uci:foreach("wireless", "wifi-iface",
		function(s)
			if s.network == net then
				m.uci:delete("wireless", s['.name'], "network")
				saved = true
			end
		end)
	if saved then m.uci:save("wireless") end
end

-- args : (wlan0, ADHOC_0)
local function wifi_set_network_by_ifname(ifname, network)
	m.uci:foreach("wireless", "wifi-iface",
		function(s)
			if s.ifname == ifname then
				m.uci:set("wireless", s['.name'], "network", network)
			end
		end)
	m.uci:save("wireless")
end

-- args : (wlan0)
local function wifi_get_interface_by_ifname(ifname)
	local intf
	m.uci:foreach("wireless", "wifi-iface",
		function(s)
			if s.ifname == ifname then
				intf = s
			end
		end)
	return intf or ""
end

local function get_network_list()
	local tbl = {}

	-- get normal interface (ethX, pppX and more)
	for _, d in ipairs(luci.sys.net.devices()) do
		-- ignore loopback, mesh device, monitor, bridge and wifi.
		if d ~= "lo" and not string.find(d, "mesh") and not nw.is_wifi(d) and
			not string.find(d, "mon.") and not string.find(d, "br-") then
			tbl[#tbl+1] = d
		end
	end

	-- get wifi interface
	m.uci:foreach("wireless", "wifi-iface",
		function(s)
			if s.ifname then
				tbl[#tbl+1] = s.ifname
			end
		end)

	return tbl
end

--------------------------------------------------------
s = m:section(NamedSection, arg[1], "interface")
s.addremove = false
s.title = arg[1]:upper()

back = s:option(DummyValue, "_overview", translate("overview"))
back.value = ""
back.titleref = luci.dispatcher.build_url("admin", "network", "network")

p = s:option(ListValue, "proto", translate("protocol"))
p.override_scheme = true
p.default = "static"
p:value("static", translate("static"))
p:value("dhcp", "DHCP")
if has_pppd  then p:value("ppp",   "PPP")     end
if has_pppoe then p:value("pppoe", "PPPoE")   end
if has_pppoa then p:value("pppoa", "PPPoA")   end
if has_3g    then p:value("3g",    "3G")      end
if has_pptp  then p:value("pptp",  "PPTP")    end
--p:value("none", translate("none"))

br = s:option(Flag, "type", translate("a_n_i_bridge"), translate("a_n_i_bridge1"))
br.enabled = "bridge"
br:depends("proto", "dhcp")
br:depends("proto", "static")
br.rmempty = true

stp = s:option(Flag, "stp", translate("a_n_i_stp"),
	translate("a_n_i_stp1", "Enables the Spanning Tree Protocol on this bridge"))
stp:depends("type", "1")
stp.rmempty = true

ifname_sif = s:option(Value, "ifname_sif", translate("network_device"))
ifname_sif.rmempty = true
ifname_sif:depends("type", "")

function ifname_sif.cfgvalue(self, section)
	local tbl = {}
	local ifname = self.map.uci:get("network", section, "ifname") or ""
	local network = ""

	-- search same network from wireless
	self.map.uci:foreach("wireless", "wifi-iface",
		function(s)
			if s.network == arg[1] then
				network = s.ifname
			end
		end)

	-- if it has wired interface, it'll show wifi-interface
	if ifname and #ifname > 0 then
		-- avoid to show multi parameters
		if type(ifname) == "string" then
			tbl = ttu.split(ifname, " ")
		end

		-- if there are two more wired interfaces, it must re-define
		-- if this network also belong to wifi-iface, it must re-define too
		if #tbl > 1 or network then
			ifname_sif.force_overwrite = true
		end

		return tbl[1]
	end

	return network
end

function ifname_sif.write(self, section, value)
	-- if value is more than two words, delete except first word
	local tbl = {}
	if type(value) == "string" then
		tbl = ttu.split(value, " ")
	end
	if #tbl > 1 then value = tbl[1] end

	-- delete same 'network' from other wifi-iface
	wifi_del_network(arg[1])
	-- delete same 'ifname' from other interface
	network_del_ifname(value)

	-- if it's wireless device, re-define 'network'
	if nw.is_wifi(value) then
		wifi_set_network_by_ifname(value, arg[1])
	else
		m.uci:set("network", section, "ifname", value)
	end
end

-- multi interfaces
ifname_multi = s:option(MultiValue, "ifname_multi", translate("network_device"))
ifname_multi.rmempty = true
ifname_multi:depends("type", "1")
ifname_multi.widget = "checkbox"
-- add values to network device list
for _, d in ipairs(get_network_list()) do
	local label = " - "
	if nw.is_wifi(d) then
		local iw = wifi_get_interface_by_ifname(d)
		local ssid = iw.ssid or "-"
		local dev = iw.device or "-"
		label = d .. label .. translate("device_wifi", "Wifi") .. " (" .. dev .. " - ESSID:" .. ssid .. ")"
	elseif nw.is_eth(d) then
		label = d .. label .. translate("device_eth", "Ethernet")
	else
		label = d
	end
	ifname_sif:value(d, label)
	ifname_multi:value(d, label)
end

function ifname_multi.cfgvalue(self, section)
	local tbl = {}
	local ifname = self.map.uci:get("network", section, "ifname") or ""

	if ifname and #ifname > 0 then
		tbl[#tbl+1] = ifname
	end

	self.map.uci:foreach("wireless", "wifi-iface",
		function(s)
			if s.network == arg[1] then
				tbl[#tbl+1] = s.ifname
			end
		end)

	return table.concat(tbl, " ")
end

function ifname_multi.write(self, section, value)
	local vtbl = {}
	if type(value) == "string" then
		vtbl = ttu.split(value, " ")
	end

	-- delete same 'network' from other wifi-iface
	wifi_del_network(arg[1])
	-- delete same 'ifname' from other interface
	for _, v in ipairs(vtbl) do
		network_del_ifname(v)
	end

	local ether = {}
	for _, v in ipairs(vtbl) do
		if nw.is_wifi(v) then
			-- set 'network' to config/wireless
			wifi_set_network_by_ifname(v, arg[1])
		else
			-- store wired device
			ether[#ether+1] = v
		end
	end

	if #ether > 0 then
		-- set 'ifname' to config/network
		m.uci:set("network", arg[1], "ifname", table.concat(ether, " "))
	end
end

local zones = wa.network_get_zones(arg[1])
if zones then
	if #zones == 0 then
		m:chain("firewall")

		fwzone = s:option(Value, "_fwzone",
			translate("network_interface_fwzone"),
			translate("network_interface_fwzone_desc"))
		fwzone.rmempty = true
		fwzone:value("", "- " .. translate("none") .. " -")
		fwzone:value(arg[1])
		m.uci:load("firewall")
		m.uci:foreach("firewall", "zone",
			function (section)
				fwzone:value(section.name)
			end
		)

		function fwzone.write(self, section, value)
			local zone = wa.firewall_find_zone(value)
			local stat

			if not zone then
				stat = m.uci:section("firewall", "zone", nil, {
					name = value,
					network = section
				})
			else
				local net = m.uci:get("firewall", zone, "network")
				net = (net or value) .. " " .. section
				stat = m.uci:set("firewall", zone, "network", net)
			end

			if stat then
				self.render = function() end
			end
		end
	else
		fwzone = s:option(DummyValue, "_fwzone", translate("firewall_zone"))
		fwzone.value = table.concat(zones, ", ")
	end
	fwzone.titleref = luci.dispatcher.build_url("admin", "network", "firewall", "zones")
	m.uci:unload("firewall")
end

ipaddr = s:option(Value, "ipaddr", translate("ipaddress"))
ipaddr.rmempty = true
ipaddr:depends("proto", "static")

mask = s:option(Value, "netmask", translate("netmask"))
mask.rmempty = true
mask:depends("proto", "static")
mask.default = "255.255.255.0"
mask:value("255.255.255.0")
mask:value("255.255.0.0")
mask:value("255.0.0.0")

gw = s:option(Value, "gateway", translate("gateway"))
gw:depends("proto", "static")
gw.rmempty = true

bcast = s:option(Value, "bcast", translate("broadcast"))
bcast:depends("proto", "static")
bcast.optional = true

--[[
ip6addr = s:option(Value, "ip6addr", translate("ip6address"), translate("cidr6"))
ip6addr.optional = true
ip6addr:depends("proto", "static")

ip6gw = s:option(Value, "ip6gw", translate("gateway6"))
ip6gw:depends("proto", "static")
ip6gw.optional = true
--]]

dns = s:option(Value, "dns", translate("dnsserver"))
dns.optional = true

mtu = s:option(Value, "mtu", "MTU")
mtu.optional = true
mtu.isinteger = true

mac = s:option(Value, "macaddr", translate("macaddress"))
mac.optional = true

srv = s:option(Value, "server", translate("network_interface_server"))
srv:depends("proto", "pptp")
srv.rmempty = true

if has_3g then
	service = s:option(ListValue, "service", translate("network_interface_service"))
	service:value("", translate("cbi_select"))
	service:value("umts", "UMTS/GPRS")
	service:value("cdma", "CDMA")
	service:value("evdo", "EV-DO")
	if has_docomo3g then
		service:value("L-02C", "L-02C")
		service:value("L-05A", "L-05A")
	end
	if has_emobile3g then
		service:value("emobile", "emobile")
	end
	service:depends("proto", "3g")
	service.rmempty = true

	apn = s:option(Value, "apn", translate("network_interface_apn"))
	if has_docomo3g then
		apn:value("mopera.net", "mopera.net (docomo, L-02C)")
		apn:value("mopera.flat.foma.ne.jp", "mopera.flat.foma.ne.jp (docomo, L-05A)")
	end
	if has_emobile3g then
		apn:value("emb.ne.jp", "emb.ne.jp (emobile)")
		apn:value("rtc.data", "rtc.data (emobile)")
	end
	apn:depends("proto", "3g")

	pincode = s:option(Value, "pincode",
	 translate("network_interface_pincode"),
	 translate("network_interface_pincode_desc")
	)
	pincode:depends("proto", "3g")
	pincode.optional = true

	pnumber = s:option(Value, "pnumber",
		translate("network_interface_pnumber"),
		translate("network_interface_pnumber_desc")
	)
	pnumber.rmempty = true
	pnumber:depends("proto", "3g")
end

if has_pppd or has_pppoe or has_pppoa or has_3g or has_pptp then
	user = s:option(Value, "username", translate("username"))
	user.rmempty = true
	user:depends("proto", "pptp")
	user:depends("proto", "pppoe")
	user:depends("proto", "pppoa")
	user:depends("proto", "ppp")
	user:depends("proto", "3g")

	pass = s:option(Value, "password", translate("password"))
	pass.rmempty = true
	pass.password = true
	pass:depends("proto", "pptp")
	pass:depends("proto", "pppoe")
	pass:depends("proto", "pppoa")
	pass:depends("proto", "ppp")
	pass:depends("proto", "3g")

	ka = s:option(Value, "keepalive",
	 translate("network_interface_keepalive"),
	 translate("network_interface_keepalive_desc")
	)
	ka.optional = true
	ka:depends("proto", "pptp")
	ka:depends("proto", "pppoe")
	ka:depends("proto", "pppoa")
	ka:depends("proto", "ppp")
	ka:depends("proto", "3g")

	demand = s:option(Value, "demand",
	 translate("network_interface_demand"),
	 translate("network_interface_demand_desc")
	)
	demand.optional = true
	demand:depends("proto", "pptp")
	demand:depends("proto", "pppoe")
	demand:depends("proto", "pppoa")
	demand:depends("proto", "ppp")
	demand:depends("proto", "3g")
end

if has_pppoa then
	encaps = s:option(ListValue, "encaps", translate("network_interface_encaps"))
	encaps.optional = false
	encaps:depends("proto", "pppoa")
	encaps:value("", translate("cbi_select"))
	encaps:value("vc", "VC")
	encaps:value("llc", "LLC")

	vpi = s:option(Value, "vpi", "VPI")
	vpi.optional = false
	vpi:depends("proto", "pppoa")

	vci = s:option(Value, "vci", "VCI")
	vci.optional = false
	vci:depends("proto", "pppoa")
end

-- this special function for peerdns and defaultroute only write the option
-- if it is set to '0' and only for 'ppp/pppoa/pppoe/pptp/3g' protocols,
-- note that this is not a valid option for 'static' and the default is '1'
function parse_default_1_func(self, section)
	local fvalue = self:formvalue(section)
	local proto = self.map:get(section, "proto")

	if fvalue then
		fvalue = self.enabled
	else
		fvalue = self.disabled
	end

	if fvalue == self.disabled and (proto == "ppp"  or proto == "pppoa" or proto == "pppoe" or proto == "pptp" or proto == "3g") then
		if not(fvalue == self:cfgvalue(section)) then
			self:write(section, fvalue)
		end
	else
		self:remove(section)
	end
end

if has_pptp or has_pppd or has_pppoe or has_pppoa or has_3g then
	device = s:option(Value, "device",
	 translate("network_interface_device"),
	 translate("network_interface_device_desc")
	)
	device:depends("proto", "ppp")
	device:depends("proto", "3g")

	defaultroute = s:option(Flag, "defaultroute",
	 translate("network_interface_defaultroute"),
	 translate("network_interface_defaultroute_desc")
	)
	defaultroute:depends("proto", "ppp")
	defaultroute:depends("proto", "pppoa")
	defaultroute:depends("proto", "pppoe")
	defaultroute:depends("proto", "pptp")
	defaultroute:depends("proto", "3g")
	defaultroute.parse = parse_default_1_func
	function defaultroute.cfgvalue(...)
		return ( AbstractValue.cfgvalue(...) or '1' )
	end

	peerdns = s:option(Flag, "peerdns",
	 translate("network_interface_peerdns"),
	 translate("network_interface_peerdns_desc")
	)
	peerdns:depends("proto", "ppp")
	peerdns:depends("proto", "pppoa")
	peerdns:depends("proto", "pppoe")
	peerdns:depends("proto", "pptp")
	peerdns:depends("proto", "3g")
	peerdns.parse = parse_default_1_func
	function peerdns.cfgvalue(...)
		return ( AbstractValue.cfgvalue(...) or '1' )
	end

--[[
	ipv6 = s:option(Flag, "ipv6", translate("network_interface_ipv6") )
	ipv6.optional = true
	ipv6:depends("proto", "ppp")
	ipv6:depends("proto", "pppoa")
	ipv6:depends("proto", "pppoe")
	ipv6:depends("proto", "pptp")
	ipv6:depends("proto", "3g")
--]]

	connect = s:option(Value, "connect",
	 translate("network_interface_connect"),
	 translate("network_interface_connect_desc")
	)
	connect.optional = true
	connect:depends("proto", "ppp")
	connect:depends("proto", "pppoe")
	connect:depends("proto", "pppoa")
	connect:depends("proto", "pptp")
	connect:depends("proto", "3g")

	disconnect = s:option(Value, "disconnect",
	 translate("network_interface_disconnect"),
	 translate("network_interface_disconnect_desc")
	)
	disconnect.optional = true
	disconnect:depends("proto", "ppp")
	disconnect:depends("proto", "pppoe")
	disconnect:depends("proto", "pppoa")
	disconnect:depends("proto", "pptp")
	disconnect:depends("proto", "3g")

	maxwait = s:option(Value, "maxwait",
	 translate("network_interface_maxwait"),
	 translate("network_interface_maxwait_desc")
	)
	maxwait.optional = true
	maxwait:depends("proto", "3g")

	pppd_options = s:option(Value, "pppd_options",
	 translate("network_interface_pppd_options"),
	 translate("network_interface_pppd_options_desc")
	)
	pppd_options.optional = true
	pppd_options:depends("proto", "ppp")
	pppd_options:depends("proto", "pppoa")
	pppd_options:depends("proto", "pppoe")
	pppd_options:depends("proto", "pptp")
	pppd_options:depends("proto", "3g")
end

s2 = m:section(TypedSection, "alias", translate("aliases"))
s2.addremove = true
s2:depends("interface", arg[1])
s2.defaults.interface = arg[1]
s2.defaults.proto = "static"
s2.cbi_add_title = translate("add_alias", "Add new IP alias")
s2.cbi_del_title = translate("del_alias", "Delete IP alias")

ipaddr_al = s2:option(Value, "ipaddr", translate("ipaddress"))
ipaddr_al.rmempty = true

mask_al = s2:option(Value, "netmask", translate("netmask"))
mask_al.rmempty = true
mask_al.default = "255.255.255.0"
mask_al:value("255.255.255.0")
mask_al:value("255.255.0.0")
mask_al:value("255.0.0.0")

gw_al = s2:option(Value, "gateway", translate("gateway"))
gw_al.rmempty = true

bcast_al = s2:option(Value, "bcast", translate("broadcast"))
bcast_al.optional = true

--[[
ip6addr_al = s2:option(Value, "ip6addr", translate("ip6address"), translate("cidr6"))
ip6addr_al.optional = true

ip6gw_al = s2:option(Value, "ip6gw", translate("gateway6"))
ip6gw_al.optional = true
--]]

dns_al = s2:option(Value, "dns", translate("dnsserver"))
dns_al.optional = true

------------------------------- validater ------------------------------
function ipaddr.validate(self, value)
	if value and #value > 0 and nw.isIPv4(value) then
		return value
	else
		self.err_message = translate("ifaces_err_addr")
		return nil
	end
end

mask.validate = ipaddr.validate
gw.validate   = ipaddr.validate
dns.validate  = ipaddr.validate

ipaddr_al.validate = ipaddr.validate
mask_al.validate   = ipaddr.validate
gw_al.validate     = ipaddr.validate
dns_al.validate    = ipaddr.validate

return m
