--[[
LuCI - Lua Configuration Interface

Copyright 2008 Steven Barth <steven@midlink.org>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

$Id$
]]--

local wa = require "luci.tools.webadmin"
local fs = require "nixio.fs"
local ttu = require "luci.tools.ttutil"

----- helper function -----
---- check wep key
-- ascii : key length should be 5(64bit) or 13(128bit)
-- hex : key length should be 10(64bit) or 26(128bit)
local function is_wepkey(key)
   if key and #key > 0 then
	  if string.match(key, "^[a-fA-F0-9]+$") and (#key == 10 or #key == 26) then
		 return true -- hex
	  elseif string.match(key, "^[a-zA-Z0-9]+$") and (#key == 5 or #key == 13) then
	     return true -- ascii
	  else
		 return false
	  end
   else
	  return false
   end
end

---- check wpa key
-- key length should be longer than 8 character and also less than 63 characters
local function is_wpakey(key)
   if key and #key > 0 then
	  if string.match(key, "^[a-zA-Z0-9]+$") and #key >= 8 and #key <= 63 then
		 return true
	  else
		 return false
	  end
   else
	  return false
   end
end
---------------------------

arg[1] = arg[1] or ""
m = Map("wireless", translate("wifi_net"), translate("wifi_net_desc"))

local phy = m:get(arg[1], "phy")
local has_11j_card = ttu.has_11j_card(phy)
local license = ttu.check_meshz_license()

s = m:section(NamedSection, arg[1], "wifi-device", translate("device") .. " " .. arg[1])
s.addremove = false

back = s:option(DummyValue, "_overview", translate("overview"))
back.value = ""
back.titleref = luci.dispatcher.build_url("admin", "network", "wireless")

en = s:option(Flag, "disabled", translate("enable_wifi"))
en.enabled = "0"
en.disabled = "1"
en.rmempty = false
function en.cfgvalue(self, section)
	return Flag.cfgvalue(self, section) or "0"
end

hwaddr = s:option(DummyValue, "_hwaddr", translate("wifi_mac_address"))
function hwaddr.cfgvalue(self, section)
	local mac = fs.readfile("/sys/class/ieee80211/" .. phy .. "/macaddress") or ""
	if mac and #mac > 0 then
		return mac:upper()
	end
	return "-"
end

s:option(DummyValue, "type", translate("type"))
local hwtype = m:get(arg[1], "type")
local nsantenna = m:get(arg[1], "antenna")

ch = s:option(ListValue, "channel", translate("a_w_channel"))
--ch:value("auto", translate("wifi_auto"), {mode="sta"})
for _, f in ipairs(luci.sys.wifi.channels(phy)) do
	if f.mhz >= 4920 and f.mhz <= 4980  then
		-- If license is empty, it will show 11a channel for just in case.
		if #license == 0 or license:find("RMR9000J") then
			ch:value(f.channel, "%i (%.3f GHz)" %{ f.channel, f.mhz / 1000 })
		end 
	else
		ch:value(f.channel, "%i (%.3f GHz)" %{ f.channel, f.mhz / 1000 })
	end
end

function ch.write(self, section, value)
	local cha = tonumber(value) or 0
	local A_band_rates = "6000 12000 24000"
	local G_band_rates = "1000 2000 5500 11000 6000 12000 24000"

	if cha >= 1 and cha <= 14 then
	   m.uci:set("wireless", section, "basic_rate", G_band_rates)
	elseif cha > 14 then
	   m.uci:set("wireless", section, "basic_rate", A_band_rates)
	end

	-- write config value
	ListValue.write(self, section, value)
end

if license == "RMR9000J-R" and has_11j_card then
	s:option(Flag, "registered_4900mhz", translate("wifi_4900MHz_registered"))
end

------------------- MAC80211 Device ------------------

if hwtype == "mac80211" then
	txpower = s:option(Value, "txpower", translate("a_w_txpwr"), "dBm")
	txpower.optional = true

--[[
	hwmode = s:option(ListValue, "hwmode", translate("mode"))
	hwmode:value("", translate("auto"))
	hwmode:value("11b", "802.11b")
	hwmode:value("11g", "802.11g")
	hwmode:value("11a", "802.11a")
	hwmode:value("11ng", "802.11g+n")
	hwmode:value("11na", "802.11a+n")
	hwmode.optional = true

	htmode = s:option(ListValue, "htmode", translate("wifi_htmode", "HT mode"))
	htmode:depends("hwmode", "11na")
	htmode:depends("hwmode", "11ng")
	htmode:value("HT20", "20MHz")
	htmode:value("HT40-", translate("wifi_ht40m", "40MHz - 2nd channel below"))
	htmode:value("HT40+", translate("wifi_ht40p", "40MHz - 2nd channel above"))
--]]

	--htcapab = s:option(DynamicList, "ht_capab", translate("wifi_htcapab", "HT capabilities"))
	--htcapab:depends("hwmode", "11na")
	--htcapab:depends("hwmode", "11ng")

	--s:option(Value, "country", translate("wifi_country", "Country Code"),
	--	translate("wifi_country_iso3166", "Use ISO/IEC 3166 alpha2 country codes.")).optional = true

	--s:option(Value, "distance", translate("wifi_distance", "Distance Optimization"),
	--	translate("wifi_distance_desc", "Distance to farthest network member in meters.")).optional = true

	ant = s:option(ListValue, "txantenna", translate("wifi_antenna", "Antenna Selection"))
	ant:value("1", translate("wifi_ant_main", "Main"))
	ant:value("2", translate("wifi_ant_aux", "Aux"))
	ant:value("3", translate("wifi_ant_both", "Both"))
	ant.optional = true

function ant.write(self, section, value)
	m.uci:set("wireless", section, "txantenna", value)
	m.uci:set("wireless", section, "rxantenna", value)
end
end


------------------- Madwifi Device ------------------

if hwtype == "atheros" then
	s:option(Value, "txpower", translate("a_w_txpwr"), "dBm").rmempty = true

	hwmode = s:option(ListValue, "hwmode", translate("mode"))
	hwmode:value("", translate("wifi_auto"))
	hwmode:value("11b", "802.11b")
	hwmode:value("11g", "802.11g")
	hwmode:value("11a", "802.11a")
	hwmode:value("11bg", "802.11b+g")
	hwmode:value("11gst", "802.11g + Turbo")
	hwmode:value("11ast", "802.11a + Turbo")
	hwmode:value("fh", translate("wifi_fh"))

	s:option(Flag, "diversity", translate("wifi_diversity")).rmempty = false

	if not nsantenna then
		s:option(Value, "txantenna", translate("wifi_txantenna")).optional = true
		s:option(Value, "rxantenna", translate("wifi_rxantenna")).optional = true
	else -- NanoFoo
		local ant = s:option(ListValue, "antenna", translate("wifi_txantenna"))
		ant:value("auto")
		ant:value("vertical")
		ant:value("horizontal")
		ant:value("external")
	end
	s:option(Value, "distance", translate("wifi_distance"),
		translate("wifi_distance_desc")).optional = true
	s:option(Value, "regdomain", translate("wifi_regdomain")).optional = true
	s:option(Value, "country", translate("wifi_country")).optional = true
	s:option(Flag, "outdoor", translate("wifi_outdoor")).optional = true

	--s:option(Flag, "nosbeacon", translate("wifi_nosbeacon")).optional = true
end



------------------- Broadcom Device ------------------

if hwtype == "broadcom" then
	s:option(Value, "txpower", translate("a_w_txpwr"), "dBm").rmempty = true

	hwmode = s:option(ListValue, "hwmode", translate("mode"))
	hwmode:value("11bg", "802.11b+g")
	hwmode:value("11b", "802.11b")
	hwmode:value("11g", "802.11g")
	hwmode:value("11gst", "802.11g + Turbo")

	mp = s:option(ListValue, "macfilter", translate("wifi_macpolicy"))
	mp.optional = true
	mp:value("")
	mp:value("allow", translate("wifi_whitelist"))
	mp:value("deny", translate("wifi_blacklist"))
	ml = s:option(DynamicList, "maclist", translate("wifi_maclist"))
	ml:depends({macfilter="allow"})
	ml:depends({macfilter="deny"})

	s:option(Value, "txantenna", translate("wifi_txantenna")).optional = true
	s:option(Value, "rxantenna", translate("wifi_rxantenna")).optional = true

	s:option(Flag, "frameburst", translate("wifi_bursting")).optional = true

	s:option(Value, "distance", translate("wifi_distance")).optional = true
	--s:option(Value, "slottime", translate("wifi_slottime")).optional = true

	s:option(Value, "country", translate("wifi_country")).optional = true
	s:option(Value, "maxassoc", translate("wifi_maxassoc")).optional = true
end


--------------------- HostAP Device ---------------------

if hwtype == "prism2" then
	s:option(Value, "txpower", translate("a_w_txpwr"), "att units").rmempty = true

	s:option(Flag, "diversity", translate("wifi_diversity")).rmempty = false

	s:option(Value, "txantenna", translate("wifi_txantenna")).optional = true
	s:option(Value, "rxantenna", translate("wifi_rxantenna")).optional = true
end


----------------------- Interface -----------------------
s = m:section(TypedSection, "wifi-iface", translate("interfaces"))
s.addremove = true
s.anonymous = true
s:depends("device", arg[1])
s.defaults.device = arg[1]
local function create_ifname_idx()
	local idxlist = {}

	-- create index list
	m.uci:foreach("wireless", "wifi-iface",
		function(s)
			if s.ifname and #s.ifname > 0 then
				local index = tonumber(s.ifname:match("wlan(%d+)"))
				idxlist[index] = index
			end
		end)

	for i=0,100 do
		if not idxlist[i] then
			return "wlan"..tostring(i)
		end
	end

	return "wlan101"
end
s.defaults.ifname = create_ifname_idx()
s.cbi_add_title = translate("wifi_add_interface", "Add new wifi interface")
s.cbi_del_title = translate("wifi_del_interface", "Delete wifi interface")

essid = s:option(Value, "ssid", translate("wifi_essid"))
essid.rmempty = false

network = s:option(Value, "network", translate("network"))
network.rmempty = true
network:value("")
network.combobox_manual = translate("a_w_netmanual")
wa.cbi_add_networks(network)

function network.write(self, section, value)
	if not m.uci:get("network", value) then
		-- avoid "value not defined in enum" because network is not known yet
		s.override_scheme = true

		m:chain("network")
		m.uci:set("network", value, "interface")
		Value.write(self, section, value)
	else
		if m.uci:get("network", value) == "interface" then
			Value.write(self, section, value)
		end
	end
end

mode = s:option(ListValue, "mode", translate("mode"))
mode.override_values = true
mode:value("ap", translate("a_w_ap"))
mode:value("adhoc", translate("a_w_adhoc"))
mode:value("sta", translate("a_w_client"))

bssid = s:option(Value, "bssid", translate("wifi_bssid"))
bssid.optional = true

-------------------- MAC80211 Interface ----------------------

if hwtype == "mac80211" then
--[[
	if fs.access("/usr/sbin/iw") then
		mode:value("mesh", "802.11s")
	end
--]]
	mode:value("monitor", translate("a_w_monitor"))
	bssid:depends({mode="adhoc"})

	--mode:value("ap-wds", "%s (WDS)" % translate("a_w_ap"))
	--mode:value("sta-wds", "%s (WDS)" % translate("a_w_client"))

	function mode.write(self, section, value)
		if value == "ap-wds" then
			ListValue.write(self, section, "ap")
			m.uci:set("wireless", section, "wds", 1)
		elseif value == "sta-wds" then
			ListValue.write(self, section, "sta")
			m.uci:set("wireless", section, "wds", 1)
		elseif value == "adhoc" then
			ListValue.write(self, section, value)
			m.uci:set("wireless", arg[1], "beacon_int", 1000)
		else
			ListValue.write(self, section, value)
			m.uci:delete("wireless", section, "wds")
			-- we need to set 'option beacon_int' only for 'adhoc' mode to change beacon interval from the default
			m.uci:delete("wireless", arg[1], "beacon_int")
		end
	end

	function mode.cfgvalue(self, section)
		local mode = ListValue.cfgvalue(self, section)
		local wds  = m.uci:get("wireless", section, "wds") == "1"

		if mode == "ap" and wds then
			return "ap-wds"
		elseif mode == "sta" and wds then
			return "sta-wds"
		else
			return mode
		end
	end

	hidden = s:option(Flag, "hidden", translate("wifi_hidden"))
	hidden:depends({mode="ap"})
	hidden:depends({mode="ap-wds"})
	hidden.optional = true

	mp = s:option(ListValue, "macfilter", translate("wifi_macpolicy"))
	mp:depends({mode="ap"})
	mp.optional = true
	mp:value("")
	mp:value("allow", translate("wifi_whitelist"))
	mp:value("deny", translate("wifi_blacklist"))
	ml = s:option(DynamicList, "maclist", translate("wifi_maclist"))
	ml:depends({macfilter="allow"})
	ml:depends({macfilter="deny"})

	s:option(Value, "frag", translate("wifi_frag")).optional = true
	s:option(Value, "rts", translate("wifi_rts")).optional = true
end



-------------------- Madwifi Interface ----------------------

if hwtype == "atheros" then
	mode:value("ahdemo", translate("a_w_ahdemo"))
	mode:value("monitor", translate("a_w_monitor"))

	bssid:depends({mode="adhoc"})
	bssid:depends({mode="ahdemo"})

	mode:value("ap-wds", "%s (WDS)" % translate("a_w_ap"))
	mode:value("sta-wds", "%s (WDS)" % translate("a_w_client"))

	function mode.write(self, section, value)
		if value == "ap-wds" then
			ListValue.write(self, section, "ap")
			m.uci:set("wireless", section, "wds", 1)
		elseif value == "sta-wds" then
			ListValue.write(self, section, "sta")
			m.uci:set("wireless", section, "wds", 1)
		else
			ListValue.write(self, section, value)
			m.uci:delete("wireless", section, "wds")
		end
	end

	function mode.cfgvalue(self, section)
		local mode = ListValue.cfgvalue(self, section)
		local wds  = m.uci:get("wireless", section, "wds") == "1"

		if mode == "ap" and wds then
			return "ap-wds"
		elseif mode == "sta" and wds then
			return "sta-wds"
		else
			return mode
		end
	end

	wdssep = s:option(Flag, "wdssep", translate("wifi_wdssep"))
	wdssep:depends({mode="ap-wds"})
	wdssep.optional = true

	s:option(Flag, "doth", "802.11h").optional = true

	hidden = s:option(Flag, "hidden", translate("wifi_hidden"))
	hidden:depends({mode="ap"})
	hidden:depends({mode="adhoc"})
	hidden:depends({mode="wds"})
	hidden:depends({mode="ap-wds"})
	hidden.optional = true

	isolate = s:option(Flag, "isolate", translate("wifi_isolate"),
	 translate("wifi_isolate_desc"))
	isolate:depends({mode="ap"})
	isolate.optional = true
	s:option(Flag, "bgscan", translate("wifi_bgscan")).optional = true

	mp = s:option(ListValue, "macfilter", translate("wifi_macfilter"))
	mp.optional = true
	mp:value("")
	mp:value("allow", translate("wifi_whitelist"))
	mp:value("deny", translate("wifi_blacklist"))
	ml = s:option(DynamicList, "maclist", translate("wifi_maclist"))
	ml:depends({macfilter="allow"})
	ml:depends({macfilter="deny"})

	s:option(Value, "rate", translate("wifi_rate")).optional = true
	s:option(Value, "mcast_rate", translate("wifi_mcast_rate")).optional = true
	s:option(Value, "frag", translate("wifi_frag")).optional = true
	s:option(Value, "rts", translate("wifi_rts")).optional = true
	s:option(Value, "minrate", translate("wifi_minrate")).optional = true
	s:option(Value, "maxrate", translate("wifi_maxrate")).optional = true
	s:option(Flag, "compression", translate("wifi_compression")).optional = true

	s:option(Flag, "bursting", translate("wifi_bursting")).optional = true
	s:option(Flag, "turbo", translate("wifi_turbo")).optional = true
	s:option(Flag, "ff", translate("wifi_ff")).optional = true

	s:option(Flag, "wmm", translate("wifi_wmm")).optional = true
	s:option(Flag, "xr", translate("wifi_xr")).optional = true
	s:option(Flag, "ar", translate("wifi_ar")).optional = true

	local swm = s:option(Flag, "sw_merge", translate("wifi_nosbeacon"))
	swm:depends({mode="adhoc"})
	swm.optional = true

	local nos = s:option(Flag, "nosbeacon", translate("wifi_nosbeacon"))
	nos:depends({mode="sta"})
	nos.optional = true

	local probereq = s:option(Flag, "probereq", translate("wifi_noprobereq"))
	probereq.optional = true
	probereq.enabled  = "0"
	probereq.disabled = "1"
end


-------------------- Broadcom Interface ----------------------

if hwtype == "broadcom" then
	mode:value("wds", translate("a_w_wds"))
	mode:value("monitor", translate("a_w_monitor"))

	hidden = s:option(Flag, "hidden", translate("wifi_hidden"))
	hidden:depends({mode="ap"})
	hidden:depends({mode="adhoc"})
	hidden:depends({mode="wds"})
	hidden.optional = true

	isolate = s:option(Flag, "isolate", translate("wifi_isolate"),
	 translate("wifi_isolate_desc"))
	isolate:depends({mode="ap"})
	isolate.optional = true

	bssid:depends({mode="wds"})
	bssid:depends({mode="adhoc"})
end


----------------------- HostAP Interface ---------------------

if hwtype == "prism2" then
	mode:value("wds", translate("a_w_wds"))
	mode:value("monitor", translate("a_w_monitor"))

	hidden = s:option(Flag, "hidden", translate("wifi_hidden"))
	hidden:depends({mode="ap"})
	hidden:depends({mode="adhoc"})
	hidden:depends({mode="wds"})
	hidden.optional = true

	bssid:depends({mode="sta"})

	mp = s:option(ListValue, "macfilter", translate("wifi_macpolicy"))
	mp.optional = true
	mp:value("")
	mp:value("allow", translate("wifi_whitelist"))
	mp:value("deny", translate("wifi_blacklist"))
	ml = s:option(DynamicList, "maclist", translate("wifi_maclist"))
	ml:depends({macfilter="allow"})
	ml:depends({macfilter="deny"})

	s:option(Value, "rate", translate("wifi_rate")).optional = true
	s:option(Value, "frag", translate("wifi_frag")).optional = true
	s:option(Value, "rts", translate("wifi_rts")).optional = true
end


------------------- WiFI-Encryption -------------------

encr = s:option(ListValue, "encryption", translate("encryption"))
encr.override_values = true
encr:depends({mode="ap"})
encr:depends({mode="sta"})
encr:depends({mode="adhoc"})
encr:depends({mode="ahdemo"})
encr:depends({mode="ap-wds"})
encr:depends({mode="sta-wds"})
encr:depends({mode="wds"})
encr:depends({mode="mesh"})

encr:value("none", translate("no_encryption", "No Encryption"))
encr:value("wep", "WEP")

if hwtype == "atheros" or hwtype == "mac80211" or hwtype == "prism2" then
	local supplicant = fs.access("/usr/sbin/wpa_supplicant")
	local hostapd = fs.access("/usr/sbin/hostapd")

	if hostapd and supplicant then
		encr:value("psk-tkip", "WPA-PSK(TKIP)", {mode="ap"}, {mode="sta"})
		encr:value("psk-aes", "WPA-PSK(AES)", {mode="ap"}, {mode="sta"})
		encr:value("psk2-tkip", "WPA2-PSK(TKIP)", {mode="ap"}, {mode="sta"})
		encr:value("psk2-aes", "WPA2-PSK(AES)", {mode="ap"}, {mode="sta"})
		encr:value("psk-mixed", translate("wpa_wpa2_mixed", "WPA-PSK/WPA2-PSK Mixed Mode"), {mode="ap"}, {mode="sta"})
--		encr:value("wpa", "WPA-EAP", {mode="ap"}, {mode="sta"})
--		encr:value("wpa2", "WPA2-EAP", {mode="ap"}, {mode="sta"})
	elseif hostapd and not supplicant then
		encr:value("psk", "WPA-PSK", {mode="ap"}, {mode="adhoc"}, {mode="ahdemo"})
		encr:value("psk2", "WPA2-PSK", {mode="ap"}, {mode="adhoc"}, {mode="ahdemo"})
		encr:value("psk-mixed", translate("wpa_wpa2_mixed", "WPA-PSK/WPA2-PSK Mixed Mode"), {mode="ap"}, {mode="adhoc"}, {mode="ahdemo"})
--		encr:value("wpa", "WPA-EAP", {mode="ap"})
--		encr:value("wpa2", "WPA2-EAP", {mode="ap"})
		encr.description = translate("wifi_wpareq")
	elseif not hostapd and supplicant then
		encr:value("psk", "WPA-PSK", {mode="sta"})
		encr:value("psk2", "WPA2-PSK", {mode="sta"})
		encr:value("psk-mixed", "WPA-PSK/WPA2-PSK Mixed Mode", {mode="sta"})
--		encr:value("wpa", "WPA-EAP", {mode="sta"})
--		encr:value("wpa2", "WPA2-EAP", {mode="sta"})
		encr.description = translate("wifi_wpareq")
	else
		encr.description = translate("wifi_wpareq")
	end
elseif hwtype == "broadcom" then
	encr:value("psk", "WPA-PSK")
	encr:value("psk2", "WPA2-PSK")
	encr:value("psk+psk2", "WPA-PSK/WPA2-PSK Mixed Mode")
end

server = s:option(Value, "server", translate("a_w_radiussrv"))
server:depends({mode="ap", encryption="wpa"})
server:depends({mode="ap", encryption="wpa2"})
server.rmempty = true

port = s:option(Value, "port", translate("a_w_radiusport"))
port:depends({mode="ap", encryption="wpa"})
port:depends({mode="ap", encryption="wpa2"})
port.rmempty = true

key = s:option(Value, "key", translate("wifi_key"))
key:depends("encryption", "wep")
key:depends("encryption", "psk") -- to keep backward compatibility
key:depends("encryption", "psk2") -- to keep backward compatibility
key:depends("encryption", "psk-tkip")
key:depends("encryption", "psk-aes")
key:depends("encryption", "psk2-tkip")
key:depends("encryption", "psk2-aes")
key:depends("encryption", "psk+psk2")
key:depends("encryption", "psk-mixed")
key:depends({mode="ap", encryption="wpa"})
key:depends({mode="ap", encryption="wpa2"})
key.rmempty = true
key.password = true

if hwtype == "atheros" or hwtype == "mac80211" or hwtype == "prism2" then
	nasid = s:option(Value, "nasid", translate("a_w_nasid"))
	nasid:depends({mode="ap", encryption="wpa"})
	nasid:depends({mode="ap", encryption="wpa2"})
	nasid.rmempty = true

	eaptype = s:option(ListValue, "eap_type", translate("a_w_eaptype"))
	eaptype:value("TLS")
	eaptype:value("TTLS")
	eaptype:value("PEAP")
	eaptype:depends({mode="sta", encryption="wpa"})
	eaptype:depends({mode="sta", encryption="wpa2"})

	cacert = s:option(FileUpload, "ca_cert", translate("a_w_cacert"))
	cacert:depends({mode="sta", encryption="wpa"})
	cacert:depends({mode="sta", encryption="wpa2"})

	privkey = s:option(FileUpload, "priv_key", translate("a_w_tlsprivkey"))
	privkey:depends({mode="sta", eap_type="TLS", encryption="wpa2"})
	privkey:depends({mode="sta", eap_type="TLS", encryption="wpa"})

	privkeypwd = s:option(Value, "priv_key_pwd", translate("a_w_tlsprivkeypwd"))
	privkeypwd:depends({mode="sta", eap_type="TLS", encryption="wpa2"})
	privkeypwd:depends({mode="sta", eap_type="TLS", encryption="wpa"})


	auth = s:option(Value, "auth", translate("a_w_peapauth"))
	auth:value("PAP")
	auth:value("CHAP")
	auth:value("MSCHAP")
	auth:value("MSCHAPV2")
	auth:depends({mode="sta", eap_type="PEAP", encryption="wpa2"})
	auth:depends({mode="sta", eap_type="PEAP", encryption="wpa"})
	auth:depends({mode="sta", eap_type="TTLS", encryption="wpa2"})
	auth:depends({mode="sta", eap_type="TTLS", encryption="wpa"})


	identity = s:option(Value, "identity", translate("a_w_peapidentity"))
	identity:depends({mode="sta", eap_type="PEAP", encryption="wpa2"})
	identity:depends({mode="sta", eap_type="PEAP", encryption="wpa"})
	identity:depends({mode="sta", eap_type="TTLS", encryption="wpa2"})
	identity:depends({mode="sta", eap_type="TTLS", encryption="wpa"})

	password = s:option(Value, "password", translate("a_w_peappassword"))
	password:depends({mode="sta", eap_type="PEAP", encryption="wpa2"})
	password:depends({mode="sta", eap_type="PEAP", encryption="wpa"})
	password:depends({mode="sta", eap_type="TTLS", encryption="wpa2"})
	password:depends({mode="sta", eap_type="TTLS", encryption="wpa"})
end

------------------------------- validater ------------------------------
function essid.validate(self, value)
	if value and #value <= 32 and string.match(value, "^[a-zA-Z0-9_%-]+$") then
		return value
	else
		self.err_message = translate("wifi_err_essid")
		return nil
	end
end

function key.validate(self, value, section)
	local encr = encr:formvalue(section) or ""
	local has_value = false

	if value and #value > 0 then
		has_value = true
	end

	-- switch by encryption type
	if encr == "wep" then
		-- wep security
		if has_value and is_wepkey(value) then
			return value
		else
			self.err_message = translate("wifi_err_wep")
			return nil
		end
	elseif encr:find("psk") then
		-- wpa security
		if has_value and is_wpakey(value) then
			return value
		else
			self.err_message = translate("wifi_err_wpa")
			return nil
		end
	elseif #encr == 0 or encr == "none" then
		-- no encryption, we don't need to check
		self.can_remove = true
		return value
	end
end

return m
